<?php

namespace App\Http\Controllers;

use Illuminate\Http\Request;
use App\model\sistema\Caja;
use App\model\sistema\Cuota;
use App\model\sistema\Oficina;
use App\model\sistema\TipoTransaccion;
use App\model\core\Common;
use Validator;

class CajaController extends Controller {

	public function __construct() {
		$this->middleware('auth');
	}

    public function index() {
    	return view('/caja', ['title' => 'CAJA']);
    }

    public function datatable_historial_caja() {
        /* $transactions = Caja::orderBy('fecha','DESC')->where('fecha','>=',date('Y-01-01'))->with('oficina', 'tipoTransaccion', 'viaje.destino')->get(); */
        $transactions = Caja::orderBy('fecha','DESC')->where('fecha','>=',date('Y-01-01', strtotime('- 1 Years')))->with('oficina', 'tipoTransaccion', 'viaje.destino')->get();
        
        $data = $transactions->map(function ($transaction) {
            return [
                'id' => $transaction->id,
                'fecha' => $transaction->fecha,
                'nro_factura' => $transaction->nro_factura,
                'concepto' => $transaction->concepto,
                'viaje' => $transaction->viaje['destino']['ciudad'],
                'fecha_viaje' => date('Y-m-d', strtotime($transaction->viaje['fecha_hora_salida'])),
                'forma_de_pago' => $transaction->tipoTransaccion['nombre'],
                'monto' => number_format($transaction->monto,2,',','.'),
                'tipo_transaccion_id' => $transaction->tipo_transaccion_id,
                'oficina' => $transaction->oficina['nombre'],
            ];
        });
    
        return response()->json(['data' => $data]);
    }

    public function getDateTransactions(Request $request) {
    	$date = $request->input('date') ? $request->input('date') : date('Y-m-d');
        $transactions = Caja::whereDate('fecha', $date)
            ->with('oficina', 'tipoTransaccion', 'viaje.destino')
            ->get();

        return $transactions->toJson();
    }

    public function getAllTransactions() {
        $transactions = Caja::orderBy('fecha','DESC')->with('oficina', 'tipoTransaccion', 'viaje.destino')->get();
    	return $transactions->toJson();
    }

    public function getDataForm(Request $request) {
        $oficinas = Oficina::all()->toJson();
        $mediosPago = TipoTransaccion::all()->toJson();
        return [$oficinas, $mediosPago];
    }

    public function attemptSaveTransaction(Request $request) {
        $rules = [
            'nro_factura' => $this->getUniqueRule($request->transaction),
            'concepto' => 'required',
            'monto' => 'required|numeric',
            'fecha' => 'required|date_format:"Y-m-d"',
            'medio_pago' => 'required',
            'oficina_id' => 'required'
        ];

        if ($this->shouldRegisterMontoCambio($request->transaction)) {
            $rules['monto_cambio'] = 'required|numeric';
        }

        $validator = Validator::make($request->transaction, $rules, []);
        if($validator->fails()) {
            return response()->json(['errors' => $validator->errors()->all()]);
        } else {
            return $this->saveTransaction($request->transaction);
        }
    }

    private function getUniqueRule($transaction) {
        return array_key_exists('id', $transaction) ? 'nullable|unique:caja2020,nro_factura,' . $transaction['id'] : 'nullable|unique:caja2020,nro_factura';
    }

    private function saveTransaction($data) {
        $transaction = array_key_exists('id', $data) ? Caja::find($data['id']) : new Caja;

        $transaction->nro_factura = $data['nro_factura'];
        $transaction->concepto = $data['concepto'];
        $transaction->monto = $data['monto'];
        $transaction->fecha = $data['fecha'];
        $transaction->tipo_transaccion_id = $data['medio_pago'];
        $transaction->metodo_pago = $data['metodo_pago'];
        $transaction->oficina_id = $data['oficina_id'];
        $transaction->id_user = auth()->id();

        if ($this->shouldRegisterMontoCambio($data)) {
            $transaction->monto_cambio = $data['monto_cambio'];
        }

        $transaction->save();

        $transaction->load('oficina', 'tipoTransaccion', 'viaje.destino');
        return $transaction;
    }

    public function removeTransaction(Request $request) {
        Cuota::where([['transaction_id', $request->id]])->delete();
        Caja::destroy($request->id);
    }

    private function shouldRegisterMontoCambio($data) {
        return array_key_exists('viaje', $data)
            && $data['viaje'] != null
            && Common::isPayingWithDifferentCurrency($data['viaje']['moneda'], $data['tipo_transaccion_id']);
    }
}
